package com.sun.corba.se.impl.util;

import org.omg.CORBA.portable.InputStream;
import org.omg.CORBA_2_3.portable.Delegate;
import org.omg.CORBA.BAD_INV_ORDER;
import org.omg.stub.java.rmi._Remote_Stub;
import org.omg.PortableServer.Servant;
import com.sun.corba.se.spi.presentation.rmi.PresentationManager;
import org.omg.CORBA.BAD_PARAM;
import org.omg.CORBA.portable.ValueFactory;
import org.omg.CORBA.CompletionStatus;
import org.omg.CORBA.portable.BoxedValueHelper;
import javax.rmi.PortableRemoteObject;
import javax.rmi.CORBA.Tie;
import org.omg.CORBA.BAD_OPERATION;
import java.rmi.RemoteException;
import org.omg.CORBA.SystemException;
import javax.rmi.CORBA.Util;
import java.rmi.Remote;
import com.sun.corba.se.spi.presentation.rmi.StubAdapter;
import org.omg.CORBA.ORB;
import com.sun.corba.se.impl.logging.OMGSystemException;
import com.sun.corba.se.impl.logging.UtilSystemException;

public final class Utility
{
    public static final String STUB_PREFIX = "_";
    public static final String RMI_STUB_SUFFIX = "_Stub";
    public static final String DYNAMIC_STUB_SUFFIX = "_DynamicStub";
    public static final String IDL_STUB_SUFFIX = "Stub";
    public static final String TIE_SUFIX = "_Tie";
    private static IdentityHashtable tieCache;
    private static IdentityHashtable tieToStubCache;
    private static IdentityHashtable stubToTieCache;
    private static Object CACHE_MISS;
    private static UtilSystemException wrapper;
    private static OMGSystemException omgWrapper;
    
    public Utility() {
    }
    
    public static Object autoConnect(Object o, ORB orb, boolean b) {
        if (o == null) {
            return o;
        }
        if (StubAdapter.isStub(o)) {
            try {
                StubAdapter.getDelegate(o);
            }
            catch (BAD_OPERATION bad_OPERATION) {
                try {
                    StubAdapter.connect(o, orb);
                }
                catch (RemoteException ex2) {
                    throw Utility.wrapper.objectNotConnected(ex2, o.getClass().getName());
                }
            }
            return o;
        }
        if ((o instanceof Remote)) {
            Tie tie = Util.getTie((Remote)o);
            if (tie != null) {
                try {
                    tie.orb();
                }
                catch (SystemException ex) {
                    tie.orb(orb);
                }
                if (b) {
                    Remote loadStub = loadStub(tie, null, null, true);
                    if (loadStub != null) {
                        return loadStub;
                    }
                    throw Utility.wrapper.couldNotLoadStub(o.getClass().getName());
                }
                else {
                    return StubAdapter.activateTie(tie);
                }
            }
            else {
                throw Utility.wrapper.objectNotExported(o.getClass().getName());
            }
        }
        else {
            return o;
        }
    }
    
    public static Tie loadTie(Remote remote) {
        Tie tie = null;
        Class<Object> clazz = remote.getClass();
        synchronized (Utility.tieCache) {
            Object value = Utility.tieCache.get(remote);
            if (value == null) {
                try {
                    for (tie = loadTie(clazz); tie == null && (clazz = clazz.getSuperclass()) != null && clazz != PortableRemoteObject.class && clazz != Object.class; tie = loadTie(clazz)) {}
                }
                catch (Exception ex2) {
                    Utility.wrapper.loadTieFailed(ex2, clazz.getName());
                }
                if (tie == null) {
                    Utility.tieCache.put(remote, Utility.CACHE_MISS);
                }
                else {
                    Utility.tieCache.put(remote, tie);
                }
            }
            else if (value != Utility.CACHE_MISS) {
                try {
                    tie = (Tie)value.getClass().newInstance();
                }
                catch (Exception ex) {}
            }
        }
        return tie;
    }
    
    private static Tie loadTie(Class clazz) {
        return com.sun.corba.se.spi.orb.ORB.getStubFactoryFactory().getTie(clazz);
    }
    
    public static void clearCaches() {
        synchronized (Utility.tieToStubCache) {
            Utility.tieToStubCache.clear();
        }
        synchronized (Utility.tieCache) {
            Utility.tieCache.clear();
        }
        synchronized (Utility.stubToTieCache) {
            Utility.stubToTieCache.clear();
        }
    }
    
    static Class loadClassOfType(String s, String s2, ClassLoader classLoader, Class clazz, ClassLoader classLoader2) throws ClassNotFoundException {
        Class<Object> clazz2 = null;
        try {
            try {
                if (!PackagePrefixChecker.hasOffendingPrefix(PackagePrefixChecker.withoutPackagePrefix(s))) {
                    clazz2 = Util.loadClass(PackagePrefixChecker.withoutPackagePrefix(s), s2, classLoader);
                }
                else {
                    clazz2 = Util.loadClass(s, s2, classLoader);
                }
            }
            catch (ClassNotFoundException ex) {
                clazz2 = Util.loadClass(s, s2, classLoader);
            }
            if (clazz == null) {
                return clazz2;
            }
        }
        catch (ClassNotFoundException ex2) {
            if (clazz == null) {
                throw ex2;
            }
        }
        if (clazz2 == null || !clazz.isAssignableFrom(clazz2)) {
            if (clazz.getClassLoader() != classLoader2) {
                throw new IllegalArgumentException("expectedTypeClassLoader not class loader of expected Type.");
            }
            if (classLoader2 != null) {
                clazz2 = classLoader2.loadClass(s);
            }
            else {
                ClassLoader classLoader3 = Thread.currentThread().getContextClassLoader();
                if (classLoader3 == null) {
                    classLoader3 = ClassLoader.getSystemClassLoader();
                }
                clazz2 = classLoader3.loadClass(s);
            }
        }
        return clazz2;
    }
    
    public static Class loadClassForClass(String name, String s, ClassLoader classLoader, Class clazz, ClassLoader classLoader2) throws ClassNotFoundException {
        if (clazz == null) {
            return Util.loadClass(name, s, classLoader);
        }
        Class<Object> clazz2 = null;
        try {
            clazz2 = Util.loadClass(name, s, classLoader);
        }
        catch (ClassNotFoundException ex) {
            if (clazz.getClassLoader() == null) {
                throw ex;
            }
        }
        if (clazz2 == null || (clazz2.getClassLoader() != null && clazz2.getClassLoader().loadClass(clazz.getName()) != clazz)) {
            if (clazz.getClassLoader() != classLoader2) {
                throw new IllegalArgumentException("relatedTypeClassLoader not class loader of relatedType.");
            }
            if (classLoader2 != null) {
                clazz2 = classLoader2.loadClass(name);
            }
        }
        return clazz2;
    }
    
    public static BoxedValueHelper getHelper(Class clazz, String codebase, String s) {
        String str = null;
        if (clazz != null) {
            str = clazz.getName();
            if (codebase == null) {
                codebase = Util.getCodebase(clazz);
            }
        }
        else {
            if (s != null) {
                str = RepositoryId.cache.getId(s).getClassName();
            }
            if (str == null) {
                throw Utility.wrapper.unableLocateValueHelper(CompletionStatus.COMPLETED_MAYBE);
            }
        }
        try {
            ClassLoader classLoader = (clazz == null) ? null : clazz.getClassLoader();
            return (BoxedValueHelper)loadClassForClass(str + "Helper", codebase, classLoader, clazz, classLoader).newInstance();
        }
        catch (ClassNotFoundException ex) {
            throw Utility.wrapper.unableLocateValueHelper(CompletionStatus.COMPLETED_MAYBE, ex);
        }
        catch (IllegalAccessException ex2) {
            throw Utility.wrapper.unableLocateValueHelper(CompletionStatus.COMPLETED_MAYBE, ex2);
        }
        catch (InstantiationException ex3) {
            throw Utility.wrapper.unableLocateValueHelper(CompletionStatus.COMPLETED_MAYBE, ex3);
        }
        catch (ClassCastException ex4) {
            throw Utility.wrapper.unableLocateValueHelper(CompletionStatus.COMPLETED_MAYBE, ex4);
        }
    }
    
    public static ValueFactory getFactory(Class clazz, String codebase, ORB orb, String s) {
        ValueFactory lookup_value_factory = null;
        if (orb != null && s != null) {
            try {
                lookup_value_factory = ((org.omg.CORBA_2_3.ORB)orb).lookup_value_factory(s);
            }
            catch (BAD_PARAM bad_PARAM) {}
        }
        String s2 = null;
        if (clazz != null) {
            s2 = clazz.getName();
            if (codebase == null) {
                codebase = Util.getCodebase(clazz);
            }
        }
        else {
            if (s != null) {
                s2 = RepositoryId.cache.getId(s).getClassName();
            }
            if (s2 == null) {
                throw Utility.omgWrapper.unableLocateValueFactory(CompletionStatus.COMPLETED_MAYBE);
            }
        }
        if (lookup_value_factory != null && (!lookup_value_factory.getClass().getName().equals(s2 + "DefaultFactory") || (clazz == null && codebase == null))) {
            return lookup_value_factory;
        }
        try {
            ClassLoader classLoader = (clazz == null) ? null : clazz.getClassLoader();
            return (ValueFactory)loadClassForClass(s2 + "DefaultFactory", codebase, classLoader, clazz, classLoader).newInstance();
        }
        catch (ClassNotFoundException ex) {
            throw Utility.omgWrapper.unableLocateValueFactory(CompletionStatus.COMPLETED_MAYBE, ex);
        }
        catch (IllegalAccessException ex2) {
            throw Utility.omgWrapper.unableLocateValueFactory(CompletionStatus.COMPLETED_MAYBE, ex2);
        }
        catch (InstantiationException ex3) {
            throw Utility.omgWrapper.unableLocateValueFactory(CompletionStatus.COMPLETED_MAYBE, ex3);
        }
        catch (ClassCastException ex4) {
            throw Utility.omgWrapper.unableLocateValueFactory(CompletionStatus.COMPLETED_MAYBE, ex4);
        }
    }
    
    public static Remote loadStub(Tie tie, PresentationManager.StubFactory stubFactory, String s, boolean b) {
        StubEntry stubEntry = null;
        synchronized (Utility.tieToStubCache) {
            Object value = Utility.tieToStubCache.get(tie);
            if (value == null) {
                stubEntry = loadStubAndUpdateCache(tie, stubFactory, s, b);
            }
            else if (value != Utility.CACHE_MISS) {
                stubEntry = (StubEntry)value;
                if (!stubEntry.mostDerived && b) {
                    stubEntry = loadStubAndUpdateCache(tie, null, s, true);
                }
                else if (stubFactory != null && !StubAdapter.getTypeIds(stubEntry.stub)[0].equals(stubFactory.getTypeIds()[0])) {
                    stubEntry = loadStubAndUpdateCache(tie, null, s, true);
                    if (stubEntry == null) {
                        stubEntry = loadStubAndUpdateCache(tie, stubFactory, s, b);
                    }
                }
                else {
                    try {
                        StubAdapter.getDelegate(stubEntry.stub);
                    }
                    catch (Exception ex) {
                        try {
                            StubAdapter.setDelegate(stubEntry.stub, StubAdapter.getDelegate(tie));
                        }
                        catch (Exception ex2) {}
                    }
                }
            }
        }
        if (stubEntry != null) {
            return (Remote)stubEntry.stub;
        }
        return null;
    }
    
    private static StubEntry loadStubAndUpdateCache(Tie tie, PresentationManager.StubFactory stubFactory, String codebase, boolean b) {
        Object o = null;
        Object o2 = null;
        int stub = StubAdapter.isStub(tie);
        if (stubFactory != null) {
            try {
                o = stubFactory.makeStub();
            }
            catch (Throwable t) {
                Utility.wrapper.stubFactoryCouldNotMakeStub(t);
                if ((t instanceof ThreadDeath)) {
                    throw (ThreadDeath)t;
                }
            }
        }
        else {
            String[] array;
            if (!(stub == 0)) {
                array = StubAdapter.getTypeIds(tie);
            }
            else {
                array = ((Servant)tie)._all_interfaces(null, null);
            }
            if (codebase == null) {
                codebase = Util.getCodebase(tie.getClass());
            }
            if (array.length == 0) {
                o = new _Remote_Stub();
            }
            else {
                int i = 0;
                while (i < array.length) {
                    if (array[i].length() == 0) {
                        o = new _Remote_Stub();
                        break;
                    }
                    try {
                        PresentationManager.StubFactoryFactory stubFactoryFactory = com.sun.corba.se.spi.orb.ORB.getStubFactoryFactory();
                        RepositoryId id = RepositoryId.cache.getId(array[i]);
                        stubFactory = stubFactoryFactory.createStubFactory(id.getClassName(), id.isIDLType(), codebase, null, tie.getClass().getClassLoader());
                        o = stubFactory.makeStub();
                        break;
                    }
                    catch (Exception ex) {
                        Utility.wrapper.errorInMakeStubFromRepositoryId(ex);
                        if (b) {
                            break;
                        }
                        ++i;
                    }
                }
            }
        }
        if (o == null) {
            Utility.tieToStubCache.put(tie, Utility.CACHE_MISS);
            return o2;
        }
        if (!(stub == 0)) {
            try {
                StubAdapter.setDelegate(o, StubAdapter.getDelegate(tie));
            }
            catch (Exception ex3) {
                synchronized (Utility.stubToTieCache) {
                    Utility.stubToTieCache.put(o, tie);
                }
            }
        }
        else {
            try {
                StubAdapter.setDelegate(o, StubAdapter.getDelegate(tie));
            }
            catch (BAD_INV_ORDER bad_INV_ORDER) {
                synchronized (Utility.stubToTieCache) {
                    Utility.stubToTieCache.put(o, tie);
                }
            }
            catch (Exception ex2) {
                throw Utility.wrapper.noPoa(ex2);
            }
        }
        o2 = new StubEntry(o, b);
        Utility.tieToStubCache.put(tie, o2);
        return o2;
    }
    
    public static Tie getAndForgetTie(org.omg.CORBA.Object object) {
        synchronized (Utility.stubToTieCache) {
            return (Tie)Utility.stubToTieCache.remove(object);
        }
    }
    
    public static void purgeStubForTie(Tie tie) {
        StubEntry stubEntry;
        synchronized (Utility.tieToStubCache) {
            stubEntry = (StubEntry)Utility.tieToStubCache.remove(tie);
        }
        if (stubEntry != null) {
            synchronized (Utility.stubToTieCache) {
                Utility.stubToTieCache.remove(stubEntry.stub);
            }
        }
    }
    
    public static void purgeTieAndServant(Tie tie) {
        synchronized (Utility.tieCache) {
            Remote target = tie.getTarget();
            if (target != null) {
                Utility.tieCache.remove(target);
            }
        }
    }
    
    public static String stubNameFromRepID(String s) {
        RepositoryId id = RepositoryId.cache.getId(s);
        String className = id.getClassName();
        if (id.isIDLType()) {
            return idlStubName(className);
        }
        return stubName(className);
    }
    
    public static Remote loadStub(org.omg.CORBA.Object object, Class clazz) {
        Remote remote = null;
        try {
            String get_codebase = null;
            try {
                get_codebase = ((Delegate)StubAdapter.getDelegate(object)).get_codebase(object);
            }
            catch (ClassCastException ex) {
                Utility.wrapper.classCastExceptionInLoadStub(ex);
            }
            remote = (Remote)com.sun.corba.se.spi.orb.ORB.getStubFactoryFactory().createStubFactory(clazz.getName(), false, get_codebase, clazz, clazz.getClassLoader()).makeStub();
            StubAdapter.setDelegate(remote, StubAdapter.getDelegate(object));
        }
        catch (Exception ex2) {
            Utility.wrapper.exceptionInLoadStub(ex2);
        }
        return remote;
    }
    
    public static Class loadStubClass(String s, String s2, Class clazz) throws ClassNotFoundException {
        if (s.length() == 0) {
            throw new ClassNotFoundException();
        }
        String stubNameFromRepID = stubNameFromRepID(s);
        ClassLoader classLoader = (clazz == null) ? null : clazz.getClassLoader();
        try {
            return loadClassOfType(stubNameFromRepID, s2, classLoader, clazz, classLoader);
        }
        catch (ClassNotFoundException ex) {
            return loadClassOfType(PackagePrefixChecker.packagePrefix() + stubNameFromRepID, s2, classLoader, clazz, classLoader);
        }
    }
    
    public static String stubName(String s) {
        return stubName(s, false);
    }
    
    public static String dynamicStubName(String s) {
        return stubName(s, true);
    }
    
    private static String stubName(String s, boolean b) {
        String str = stubNameForCompiler(s, b);
        if (PackagePrefixChecker.hasOffendingPrefix(str)) {
            str = PackagePrefixChecker.packagePrefix() + str;
            return str;
        }
        return str;
    }
    
    public static String stubNameForCompiler(String s) {
        return stubNameForCompiler(s, false);
    }
    
    private static String stubNameForCompiler(String str, boolean b) {
        int n = str.indexOf(36);
        if (n < 0) {
            n = str.lastIndexOf(46);
        }
        String s = b ? "_DynamicStub" : "_Stub";
        if (n > 0) {
            return str.substring(0, n + 1) + "_" + str.substring(n + 1) + s;
        }
        return "_" + str + s;
    }
    
    public static String tieName(String s) {
        return PackagePrefixChecker.hasOffendingPrefix(tieNameForCompiler(s)) ? (PackagePrefixChecker.packagePrefix() + tieNameForCompiler(s)) : tieNameForCompiler(s);
    }
    
    public static String tieNameForCompiler(String str) {
        int n = str.indexOf(36);
        if (n < 0) {
            n = str.lastIndexOf(46);
        }
        if (n > 0) {
            return str.substring(0, n + 1) + "_" + str.substring(n + 1) + "_Tie";
        }
        return "_" + str + "_Tie";
    }
    
    public static void throwNotSerializableForCorba(String s) {
        throw Utility.omgWrapper.notSerializable(CompletionStatus.COMPLETED_MAYBE, s);
    }
    
    public static String idlStubName(String str) {
        int lastIndex = str.lastIndexOf(46);
        if (lastIndex > 0) {
            return str.substring(0, lastIndex + 1) + "_" + str.substring(lastIndex + 1) + "Stub";
        }
        return "_" + str + "Stub";
    }
    
    public static void printStackTrace() {
        Throwable t = new Throwable("Printing stack trace:");
        t.fillInStackTrace();
        t.printStackTrace();
    }
    
    public static Object readObjectAndNarrow(InputStream inputStream, Class clazz) throws ClassCastException {
        org.omg.CORBA.Object read_Object = inputStream.read_Object();
        if (read_Object != null) {
            return PortableRemoteObject.narrow(read_Object, clazz);
        }
        return null;
    }
    
    public static Object readAbstractAndNarrow(org.omg.CORBA_2_3.portable.InputStream inputStream, Class clazz) throws ClassCastException {
        Object read_abstract_interface = inputStream.read_abstract_interface();
        if (read_abstract_interface != null) {
            return PortableRemoteObject.narrow(read_abstract_interface, clazz);
        }
        return null;
    }
    
    static int hexOf(char c) {
        int n = c - 48;
        if (n >= 0 && n <= 9) {
            return n;
        }
        int n2 = c - 97 + 10;
        if (n2 >= 10 && n2 <= 15) {
            return n2;
        }
        int n3 = c - 65 + 10;
        if (n3 >= 10 && n3 <= 15) {
            return n3;
        }
        throw Utility.wrapper.badHexDigit();
    }
    
    static {
        Utility.tieCache = new IdentityHashtable();
        Utility.tieToStubCache = new IdentityHashtable();
        Utility.stubToTieCache = new IdentityHashtable();
        Utility.CACHE_MISS = new Object();
        Utility.wrapper = UtilSystemException.get("util");
        Utility.omgWrapper = OMGSystemException.get("util");
    }
}
